﻿Option Strict On
Option Explicit On

' Copyright © exc-jdbi 2021
' © Finite Field  2012
' © Finite Field  2012 Is Free Software
' https://www.vb-paradise.de/index.php/User/23082-exc-jdbi/
' github.com/exc-jdbi


Imports exc.jdbi.FiniteFieldArithmetics

'Für diesen Test stimmen die irreduziblen Polynomial

'Werden weitere GF-Körper getestet, dann immer mehrere
'irreduzible Polynomial testen. Im Normalfall ist die 
'Teilerfremdeit aller Zahlen gewährleistet. Es kann aber 
'vorkommen, dass gewisse irreduzible Polynomial das 
'nicht erreichen, in dem Falle einfach das nächste
'irreduzible Polynomial ausprobieren.

'WolframAlpha ist ein geeignetes Instrument um 
'irreduzible Polynomial zu finden. Auch hier werden
'meistens mehrere Vorschläge gebracht.
'https://www.wolframalpha.com/input/?i=GF%283%5E16%29


Public Module TestFFOperations
  Private ReadOnly Generators() As Int32 = {2, 3, 5, 7, 11, 13, 17}
  Private ReadOnly ExponentsMax() As Int32 = {8, 5, 4, 3, 2, 2, 2}

  'Irreduzible Polynomial
  Private ReadOnly IrrDuzPoly_Gf_2() As Int32 = {1, 7, 11, 19, 37, 67, 131, 285}
  Private ReadOnly IrrDuzPoly_Gf_3() As Int32 = {1, 10, 34, 86, 250}
  Private ReadOnly IrrDuzPoly_Gf_5() As Int32 = {1, 27, 131, 627}
  Private ReadOnly IrrDuzPoly_Gf_7() As Int32 = {1, 50, 345}
  Private ReadOnly IrrDuzPoly_Gf_11() As Int32 = {1, 122}
  Private ReadOnly IrrDuzPoly_Gf_13() As Int32 = {1, 171}
  Private ReadOnly IrrDuzPoly_Gf_17() As Int32 = {1, 292}


  Public Sub Main()
    TestFFSystemOperation()

  End Sub

  Public Sub TestFFSystemOperation()
    'Testdauer ca. 5 - 10 min.

    Dim sw = Stopwatch.StartNew

    Dim allidp =
    {
      IrrDuzPoly_Gf_2, IrrDuzPoly_Gf_3, IrrDuzPoly_Gf_5, IrrDuzPoly_Gf_7,
      IrrDuzPoly_Gf_11, IrrDuzPoly_Gf_13, IrrDuzPoly_Gf_17
    }

    For i = 0 To Generators.Length - 1
      Dim idp = allidp(i)
      Dim gen = Generators(i)
      Dim exp = ExponentsMax(i)
      For j = 1 To exp
        Dim gfx = New FiniteField(gen, j, idp(j - 1), 0)

        TestFFSystemOperation(gfx)
        Console.WriteLine()
      Next
    Next

    Console.WriteLine()
    Console.WriteLine($"t = {sw.ElapsedMilliseconds}ms")
    Console.WriteLine()
    Console.WriteLine($"TestFFSystemOperation abgeschlossen. I.O.")
    Console.WriteLine()

    Console.ReadLine()
  End Sub

  Private Sub TestFFSystemOperation(gfx As FiniteField)

    Dim gfx1 = gfx.Copy
    Dim gfx2 = gfx.Copy
    Dim gfx3 As FiniteField = Nothing
    Dim testtmp = New Boolean(CType(gfx.Order, Int32) - 1) {}

    'Additionsprüfung
    For i = 0 To gfx.Order - 1
      gfx1.Value = i
      Array.Clear(testtmp, 0, testtmp.Length)
      For j = 0 To gfx.Order - 1
        gfx2.Value = j
        gfx3 = gfx1 + gfx2
        If testtmp(CType(gfx3.Value, Int32)) Then
          Console.WriteLine($"Fehler in: i = {i}; j = {j}; g = {gfx.Generator}, e = {gfx.Exponent}")
        End If
        testtmp(CType(gfx3.Value, Int32)) = True
      Next
      If testtmp.Contains(False) Then
        Console.WriteLine($"Fehler in: i = {i}; g = {gfx.Generator}, e = {gfx.Exponent}; Es gibt noch falses")
      End If
    Next
    Console.WriteLine($"Additionsprüfung getestet g = {gfx.Generator}, e = {gfx.Exponent}")

    'Multiplikationsprüfung
    For i = 1 To gfx.Order - 1
      gfx1.Value = i
      Array.Clear(testtmp, 0, testtmp.Length)
      For j = 0 To gfx.Order - 1
        gfx2.Value = j
        gfx3 = gfx1 * gfx2
        If testtmp(CType(gfx3.Value, Int32)) Then
          Console.WriteLine($"Fehler in: i = {i}; j = {j}; g = {gfx.Generator}, e = {gfx.Exponent}")
        End If
        testtmp(CType(gfx3.Value, Int32)) = True
      Next
      If testtmp.Contains(False) Then
        Console.WriteLine($"Fehler in: i = {i}; g = {gfx.Generator}, e = {gfx.Exponent}; Es gibt noch falses")
      End If
    Next
    Console.WriteLine($"Multiplikationsprüfung getestet g = {gfx.Generator}, e = {gfx.Exponent}")

    'Subtraktionsprüfung
    For i = 0 To gfx.Order - 1
      gfx1.Value = i
      Array.Clear(testtmp, 0, testtmp.Length)
      For j = 0 To gfx.Order - 1
        gfx2.Value = j
        gfx3 = gfx1 - gfx2
        If testtmp(CType(gfx3.Value, Int32)) Then
          Console.WriteLine($"Fehler in: i = {i}; j = {j}; g = {gfx.Generator}, e = {gfx.Exponent}")
        End If
        testtmp(CType(gfx3.Value, Int32)) = True
      Next
      If testtmp.Contains(False) Then
        Console.WriteLine($"Fehler in: i = {i}; g = {gfx.Generator}, e = {gfx.Exponent}; Es gibt noch falses")
      End If
    Next
    Console.WriteLine($"Subtraktionsprüfung getestet g = {gfx.Generator}, e = {gfx.Exponent}")


    'Divisionsprüfung
    For i = 1 To gfx.Order - 1
      gfx1.Value = i
      Array.Clear(testtmp, 0, testtmp.Length)
      testtmp(0) = True
      For j = 1 To gfx.Order - 1
        gfx2.Value = j
        gfx3 = gfx1 / gfx2
        If testtmp(CType(gfx3.Value, Int32)) Then
          Console.WriteLine($"Fehler in: i = {i}; j = {j}; g = {gfx.Generator}, e = {gfx.Exponent}")
        End If
        testtmp(CType(gfx3.Value, Int32)) = True
      Next
      If testtmp.Contains(False) Then
        Console.WriteLine($"Fehler in: i = {i}; g = {gfx.Generator}, e = {gfx.Exponent}; Es gibt noch falses")
      End If
    Next
    Console.WriteLine($"Divisionsprüfung getestet g = {gfx.Generator}, e = {gfx.Exponent}")

  End Sub


End Module
